@@ -1,12 +1,13 @@
 # This file is part of ranger, the console file manager.
 # License: GNU GPL version 3, see the file "AUTHORS" for details.
 
+from __future__ import (absolute_import, division, print_function)
+
 import locale
 import os.path
+from os import stat as os_stat, lstat as os_lstat
 import random
 import re
-
-from os import stat as os_stat, lstat as os_lstat
 from collections import deque
 from time import time
 
@@ -56,17 +57,17 @@ def sort_unicode_wrapper_list(old_sort_func):
     return sort_unicode
 
 
-def accept_file(file, filters):
+def accept_file(fobj, filters):
     """
     Returns True if file shall be shown, otherwise False.
     Parameters:
-        file - an instance of FileSystemObject
-        filters - an array of lambdas, each expects a file and
-                  returns True if file shall be shown,
+        fobj - an instance of FileSystemObject
+        filters - an array of lambdas, each expects a fobj and
+                  returns True if fobj shall be shown,
                   otherwise False.
     """
-    for filter in filters:
-        if filter and not filter(file):
+    for filt in filters:
+        if filt and not filt(fobj):
             return False
     return True
 
@@ -85,14 +86,21 @@ def walklevel(some_dir, level):
 
 def mtimelevel(path, level):
     mtime = os.stat(path).st_mtime
-    for dirpath, dirnames, filenames in walklevel(path, level):
+    for dirpath, dirnames, _ in walklevel(path, level):
         dirlist = [os.path.join("/", dirpath, d) for d in dirnames
-                if level == -1 or dirpath.count(os.path.sep) - path.count(os.path.sep) <= level]
+                   if level == -1 or dirpath.count(os.path.sep) - path.count(os.path.sep) <= level]
         mtime = max(mtime, max([-1] + [os.stat(d).st_mtime for d in dirlist]))
     return mtime
 
 
-class Directory(FileSystemObject, Accumulator, Loadable):
+class InodeFilterConstants(object):  # pylint: disable=too-few-public-methods
+    DIRS = 'd'
+    FILES = 'f'
+    LINKS = 'l'
+
+
+class Directory(  # pylint: disable=too-many-instance-attributes,too-many-public-methods
+        FileSystemObject, Accumulator, Loadable):
     is_directory = True
     enterable = False
     load_generator = None
@@ -106,6 +114,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     files_all = None
     filter = None
     temporary_filter = None
+    narrow_filter = None
     inode_type_filter = None
     marked_items = None
     scroll_begin = 0
@@ -120,10 +129,10 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     content_outdated = False
     content_loaded = False
 
-    vcs = None
     has_vcschild = False
+    _vcs_signal_handler_installed = False
 
-    _cumulative_size_calculated = False
+    cumulative_size_calculated = False
 
     sort_dict = {
         'basename': sort_by_basename,
@@ -144,24 +153,42 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         Accumulator.__init__(self)
         FileSystemObject.__init__(self, path, **kw)
 
-        self.marked_items = list()
-
-        for opt in ('sort_directories_first', 'sort', 'sort_reverse',
-                'sort_case_insensitive'):
-            self.settings.signal_bind('setopt.' + opt,
-                    self.request_resort, weak=True, autosort=False)
+        self.marked_items = []
 
+        self._signal_functions = []
+        func = self.signal_function_factory(self.sort)
+        self._signal_functions += [func]
+        for opt in ('sort_directories_first', 'sort', 'sort_reverse', 'sort_case_insensitive'):
+            self.settings.signal_bind('setopt.' + opt, func, weak=True, autosort=False)
+        func = self.signal_function_factory(self.refilter)
+        self._signal_functions += [func]
         for opt in ('hidden_filter', 'show_hidden'):
-            self.settings.signal_bind('setopt.' + opt,
-                self.refilter, weak=True, autosort=False)
+            self.settings.signal_bind('setopt.' + opt, func, weak=True, autosort=False)
 
         self.settings = LocalSettings(path, self.settings)
 
-        if self.settings.vcs_aware:
-            self.vcs = Vcs(self)
-
         self.use()
 
+    @lazy_property
+    def vcs(self):
+        if not self._vcs_signal_handler_installed:
+            self.settings.signal_bind(
+                'setopt.vcs_aware', self.vcs__reset,  # pylint: disable=no-member
+                weak=True, autosort=False,
+            )
+            self._vcs_signal_handler_installed = True
+        if self.settings.vcs_aware:
+            return Vcs(self)
+        return None
+
+    def signal_function_factory(self, function):
+        def signal_function():
+            self.load_if_outdated()
+            if not self.exists:
+                return
+            function()
+        return signal_function
+
     def request_resort(self):
         self.order_outdated = True
 
@@ -172,7 +199,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         return self.files
 
     def mark_item(self, item, val):
-        item._mark(val)
+        item.mark_set(val)
         if val:
             if item in self.files and item not in self.marked_items:
                 self.marked_items.append(item)
@@ -207,7 +234,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
     def _clear_marked_items(self):
         for item in self.marked_items:
-            item._mark(False)
+            item.mark_set(False)
         del self.marked_items[:]
 
     def get_selection(self):
@@ -215,12 +242,13 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         self._gc_marked_items()
         if not self.files:
             return []
+
         if self.marked_items:
             return [item for item in self.files if item.marked]
         elif self.pointed_obj:
             return [self.pointed_obj]
-        else:
-            return []
+
+        return []
 
     def refilter(self):
         if self.files_all is None:
@@ -233,15 +261,42 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         if not self.settings.show_hidden and self.settings.hidden_filter:
             hidden_filter = re.compile(self.settings.hidden_filter)
             hidden_filter_search = hidden_filter.search
-            filters.append(lambda file: not hidden_filter_search(file.basename))
+
+            def hidden_filter_func(fobj):
+                for comp in fobj.relative_path.split(os.path.sep):
+                    if hidden_filter_search(comp):
+                        return False
+                return True
+            filters.append(hidden_filter_func)
+        if self.narrow_filter:
+            # pylint: disable=unsupported-membership-test
+
+            # Pylint complains that self.narrow_filter is by default
+            # None but the execution won't reach this line if it is
+            # still None.
+            filters.append(lambda fobj: fobj.basename in self.narrow_filter)
+        if self.settings.global_inode_type_filter or self.inode_type_filter:
+            def inode_filter_func(obj):
+                # Use local inode_type_filter if present, global otherwise
+                inode_filter = self.inode_type_filter or self.settings.global_inode_type_filter
+                # Apply filter
+                if InodeFilterConstants.DIRS in inode_filter and \
+                        obj.is_directory:
+                    return True
+                elif InodeFilterConstants.FILES in inode_filter and \
+                        obj.is_file and not obj.is_link:
+                    return True
+                elif InodeFilterConstants.LINKS in inode_filter and \
+                        obj.is_link:
+                    return True
+                return False
+            filters.append(inode_filter_func)
         if self.filter:
             filter_search = self.filter.search
-            filters.append(lambda file: filter_search(file.basename))
-        if self.inode_type_filter:
-            filters.append(self.inode_type_filter)
+            filters.append(lambda fobj: filter_search(fobj.basename))
         if self.temporary_filter:
             temporary_filter_search = self.temporary_filter.search
-            filters.append(lambda file: temporary_filter_search(file.basename))
+            filters.append(lambda fobj: temporary_filter_search(fobj.basename))
 
         self.files = [f for f in self.files_all if accept_file(f, filters)]
 
@@ -256,6 +311,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         self.move_to_obj(self.pointed_obj)
 
     # XXX: Check for possible race conditions
+    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
     def load_bit_by_bit(self):
         """An iterator that loads a part on every next() call
 
@@ -269,7 +325,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
         basename_is_rel_to = self.path if self.flat else None
 
-        try:
+        try:  # pylint: disable=too-many-nested-blocks
             if self.runnable:
                 yield
                 mypath = self.path
@@ -279,8 +335,12 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                 if self.flat:
                     filelist = []
                     for dirpath, dirnames, filenames in walklevel(mypath, self.flat):
-                        dirlist = [os.path.join("/", dirpath, d) for d in dirnames
-                                if self.flat == -1 or dirpath.count(os.path.sep) - mypath.count(os.path.sep) <= self.flat]
+                        dirlist = [
+                            os.path.join("/", dirpath, d)
+                            for d in dirnames
+                            if self.flat == -1 or
+                            (dirpath.count(os.path.sep) - mypath.count(os.path.sep)) <= self.flat
+                        ]
                         filelist += dirlist
                         filelist += [os.path.join("/", dirpath, f) for f in filenames]
                     filenames = filelist
@@ -288,10 +348,10 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                 else:
                     filelist = os.listdir(mypath)
                     filenames = [mypath + (mypath == '/' and fname or '/' + fname)
-                            for fname in filelist]
+                                 for fname in filelist]
                     self.load_content_mtime = os.stat(mypath).st_mtime
 
-                if self._cumulative_size_calculated:
+                if self.cumulative_size_calculated:
                     # If self.content_loaded is true, this is not the first
                     # time loading.  So I can't really be sure if the
                     # size has changed and I'll add a "?".
@@ -324,39 +384,43 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                             file_stat = os_stat(name)
                         else:
                             file_stat = file_lstat
+                    except OSError:
+                        file_lstat = None
+                        file_stat = None
+                    if file_lstat and file_stat:
                         stats = (file_stat, file_lstat)
                         is_a_dir = file_stat.st_mode & 0o170000 == 0o040000
-                    except Exception:
+                    else:
                         stats = None
                         is_a_dir = False
+
                     if is_a_dir:
-                        try:
-                            item = self.fm.get_directory(name)
-                            item.load_if_outdated()
-                        except Exception:
-                            item = Directory(name, preload=stats, path_is_abs=True,
-                                             basename_is_rel_to=basename_is_rel_to)
-                            item.load()
+                        item = self.fm.get_directory(name, preload=stats, path_is_abs=True,
+                                                     basename_is_rel_to=basename_is_rel_to)
+                        item.load_if_outdated()
+                        if self.flat:
+                            item.relative_path = os.path.relpath(item.path, self.path)
                         else:
-                            if self.flat:
-                                item.relative_path = os.path.relpath(item.path, self.path)
-                            else:
-                                item.relative_path = item.basename
-                            item.relative_path_lower = item.relative_path.lower()
+                            item.relative_path = item.basename
+                        item.relative_path_lower = item.relative_path.lower()
                         if item.vcs and item.vcs.track:
                             if item.vcs.is_root_pointer:
                                 has_vcschild = True
                             else:
-                                item.vcsstatus = item.vcs.rootvcs.status_subpath(
-                                    os.path.join(self.realpath, item.basename), is_directory=True)
+                                item.vcsstatus = \
+                                    item.vcs.rootvcs.status_subpath(  # pylint: disable=no-member
+                                        os.path.join(self.realpath, item.basename),
+                                        is_directory=True,
+                                    )
                     else:
                         item = File(name, preload=stats, path_is_abs=True,
                                     basename_is_rel_to=basename_is_rel_to)
                         item.load()
                         disk_usage += item.size
                         if self.vcs and self.vcs.track:
-                            item.vcsstatus = self.vcs.rootvcs.status_subpath(
-                                os.path.join(self.realpath, item.basename))
+                            item.vcsstatus = \
+                                self.vcs.rootvcs.status_subpath(  # pylint: disable=no-member
+                                    os.path.join(self.realpath, item.basename))
 
                     files.append(item)
                     self.percent = 100 * len(files) // len(filenames)
@@ -370,10 +434,10 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                 self._clear_marked_items()
                 for item in self.files_all:
                     if item.path in marked_paths:
-                        item._mark(True)
+                        item.mark_set(True)
                         self.marked_items.append(item)
                     else:
-                        item._mark(False)
+                        item.mark_set(False)
 
                 self.sort()
 
@@ -397,6 +461,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             self.fm.signal_emit("finished_loading_dir", directory=self)
             if self.vcs:
                 self.fm.ui.vcsthread.process(self)
+    # pylint: enable=too-many-locals,too-many-branches,too-many-statements
 
     def unload(self):
         self.loading = False
@@ -408,6 +473,8 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         Use this sparingly since it takes rather long.
         """
         self.content_outdated = False
+        if self.settings.freeze_files:
+            return
 
         if not self.loading:
             if not self.loaded:
@@ -442,7 +509,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
         try:
             sort_func = self.sort_dict[self.settings.sort]
-        except Exception:
+        except KeyError:
             sort_func = sort_by_basename
 
         if self.settings.sort_case_insensitive and \
@@ -475,27 +542,25 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             return 0
         cum = 0
         realpath = os.path.realpath
-        for dirpath, dirnames, filenames in os.walk(self.path,
-                onerror=lambda _: None):
-            for file in filenames:
+        for dirpath, _, filenames in os.walk(self.path, onerror=lambda _: None):
+            for fname in filenames:
                 try:
                     if dirpath == self.path:
-                        stat = os_stat(realpath(dirpath + "/" + file))
+                        stat = os_stat(realpath(dirpath + "/" + fname))
                     else:
-                        stat = os_stat(dirpath + "/" + file)
-                    cum += stat.st_size
-                except Exception:
-                    pass
+                        stat = os_stat(dirpath + "/" + fname)
+                except OSError:
+                    continue
+                cum += stat.st_size
         return cum
 
     def look_up_cumulative_size(self):
-        self._cumulative_size_calculated = True
+        self.cumulative_size_calculated = True
         self.size = self._get_cumulative_size()
-        self.infostring = ('-> ' if self.is_link else ' ') + \
-                human_readable(self.size)
+        self.infostring = ('-> ' if self.is_link else ' ') + human_readable(self.size)
 
     @lazy_property
-    def size(self):
+    def size(self):  # pylint: disable=method-hidden
         try:
             if self.fm.settings.automatically_count_files:
                 size = len(os.listdir(self.path))
@@ -516,15 +581,15 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             return size
 
     @lazy_property
-    def infostring(self):
-        self.size  # trigger the lazy property initializer
+    def infostring(self):  # pylint: disable=method-hidden
+        self.size  # trigger the lazy property initializer pylint: disable=pointless-statement
         if self.is_link:
             return '->' + self.infostring
         return self.infostring
 
     @lazy_property
-    def runnable(self):
-        self.size  # trigger the lazy property initializer
+    def runnable(self):  # pylint: disable=method-hidden
+        self.size  # trigger the lazy property initializer pylint: disable=pointless-statement
         return self.runnable
 
     def sort_if_outdated(self):
@@ -535,10 +600,10 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             return True
         return False
 
-    def move_to_obj(self, arg):
+    def move_to_obj(self, arg, attr=None):
         try:
             arg = arg.path
-        except Exception:
+        except AttributeError:
             pass
         self.load_content_once(schedule=False)
         if self.empty():
@@ -547,17 +612,13 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         Accumulator.move_to_obj(self, arg, attr='path')
 
     def search_fnc(self, fnc, offset=1, forward=True):
-        if not hasattr(fnc, '__call__'):
-            return False
-
         length = len(self)
-
         if forward:
             generator = ((self.pointer + (x + offset)) % length
-                    for x in range(length - 1))
+                         for x in range(length - 1))
         else:
             generator = ((self.pointer - (x + offset)) % length
-                    for x in range(length - 1))
+                         for x in range(length - 1))
 
         for i in generator:
             _file = self.files[i]
@@ -584,11 +645,11 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         """Make sure the pointer is in the valid range"""
         Accumulator.correct_pointer(self)
 
-        try:
-            if self == self.fm.thisdir:
+        if self == self.fm.thisdir:
+            try:
                 self.fm.thisfile = self.pointed_obj
-        except Exception:
-            pass
+            except AttributeError:
+                pass
 
     def load_content_once(self, *a, **k):
         """Load the contents of the directory if not done yet"""
@@ -638,7 +699,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             return True
         return self.last_used + seconds < time()
 
-    def go(self, history=True):
+    def go(self, history=True):  # pylint: disable=invalid-name
         """enter the directory if the filemanager is running"""
         if self.fm:
             return self.fm.enter_dir(self.path, history=history)
@@ -646,11 +707,11 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
     def empty(self):
         """Is the directory empty?"""
-        return self.files is None or len(self.files) == 0
+        return not self.files
 
-    def _set_linemode_of_children(self, mode):
-        for f in self.files:
-            f._set_linemode(mode)
+    def set_linemode_of_children(self, mode):
+        for fobj in self.files:
+            fobj.set_linemode(mode)
 
     def __nonzero__(self):
         """Always True"""
